
# ----------------------------------------------------------------

# {{{ code: Package Initialization
package Text::Extract;
# }}} ------------------------------------------------------------
# {{{ docs: perldocs

=pod

=head1 NAME

Text::Extract - Extract strings/values from text.

=head1 SYNOPSIS

  use Text::Extract;
  my( $format ) = Text::Extract->new( "%d. %s" );
  print( map( "%d is %s\n", $format->GetValues( "4. kebob\n" ) ) );

=head1 DESCRIPTION

Extract tokens from given text.  By default, the format of the text is
given with printf-like format symbols.  %s defines a free-form string,
%d, %i, etc define digits, %f defines a float, etc.  Essentially, this
does what scanf(3) does.  One bonus, though, is that you can define your
own list of symbols, giving each an associated. regular expression.

=cut

# }}}
# {{{ code: Package Variables
use vars qw( %definedFormatMaps );

%definedFormatMaps =
  ( 'default' => { s => '\s+',
		   d => '-?\d+',
		   D => '-?\d+',
		   i => [ '-?\d+', '0[xX][\da-fA-F]+', '0[0-7]+' ],
		   V => '-?\d+',
		   u => '\d+',
		   U => '\d+',
		   o => '[0-7]+',
		   O => '[0-7]+',
		   x => '[\da-f]+',
		   X => '[\dA-F]+',
		   e => '[\d.]+e\[+-]\d+',
		   E => '[\d.]+E\[+-]\d+',
		   f => '[\d.]+',
		   F => '[\d.]+',
		   c => '.' },
  );

# }}} ------------------------------------------------------------
# {{{ code: Object Methods

# {{{  perldocs

=head1 METHODS

=cut

# }}}

# {{{  sub: new
# {{{   perldocs

=item new( [$format|%format_map] )

Specify either a format to use with the default formatting symbols, or
a new set of formating symbol mapping to replace the default set with.

=cut

# }}}

sub new {
    my( $class  ) = shift;
    my( $format, %format_map, $self );

    $self = {};
    if( ref( $class ) ) {
	$format = $class->Format;
	%format_map = $class->FormatMap;
	$class = ref( $class );
    }

    if( scalar( @_ ) ) {

	$format = shift;

    }

    if( scalar( @_ ) > 1 ) {

	%format_map = @_;

    } else {

	my( $name ) = shift || 'default';
	%format_map = ( exists( $definedFormatMaps{ $name } )
		       ? %{ $definedFormatMaps{ $name } }
		       : %{ $definedFormatMaps{ 'default' } } );

    }

    bless( $self, $class );

    $self->FormatMap( %format_map );
    $self->Format( $format );

    return( $self );

}

# }}} ------------------------------------------------------------
# {{{  sub: FormatMap
# {{{   perldoc

=item FormatMap( %format_map )

Set the format mapping to use with this Extract object.

=cut

# }}}

sub FormatMap {
    my( $self ) = shift;

    if( scalar( @_ ) ) {
	$self->{ FormatMap } = { @_ };
	if( defined( $self->Format ) ) {
	    $self->_RE( _convert_format_to_re( $self->Format, $self->FormatMap ) );
	    $self->FormatOrder( _get_format_order( $self->Format, $self->FormatMap ) );
	}
    }

    return( %{ $self->{ FormatMap } } );
}

# }}} ------------------------------------------------------------
# {{{  sub: Format
# {{{   perldoc

=item Format

Set the format string to use to parse text.

=cut

# }}}

sub Format {
    my( $self ) = shift;

    if( scalar( @_ ) ) {
	$self->{ Format } = shift;
	if( $self->FormatMap ) {
	    $self->_RE( _convert_format_to_re( $self->Format, $self->FormatMap ) );
	    $self->FormatOrder( _get_format_order( $self->Format, $self->FormatMap ) );
	}
    }

    return( $self->{ Format } );
}

# }}} ------------------------------------------------------------
# {{{  sub: FormatOrder
# {{{   perldoc

=item FormatOrder

FormatOrder is an ordered list of symbols as extracted from the format
string.  It is stored here for convenience, basically a cacheing mechanism.

=cut

# }}}

sub FormatOrder {
    my( $self ) = shift;

    $self->{ FormatOrder } = [ @_ ]
      if( scalar( @_ ) );

    return( @{ $self->{ FormatOrder } } );
}

# }}} ------------------------------------------------------------
# {{{  sub: GetValues
# {{{   perldoc

=item  GetValues( $string )

Extract values from the given string useing the stored Format string.
This returns a hash mapping of symbols to values.  This function is not
suitable if you have multiple identical symbols in your format string as
once these are put into the hash, only one symbol of each type will be
present.  Use GetOrderedValues if you have multiple identical symbols.

=cut

# }}}

sub GetValues {
    my( $self  ) = shift;
    my( $string ) = shift;
    my( $re );

    $re = $self->_RE();
    @values{ $self->FormatOrder } = ( $string =~ m/$re/ );

    return( %values );

}

# }}} ------------------------------------------------------------
# {{{  sub: GetOrderedValues
# {{{   perldoc

=item GetOrderedValues( $string )

Extract values from the given string useing the stored Format string.
This returns a list of values from the string in the same order that
they are found in $string.

=cut

# }}}

sub GetOrderedValues {
    my( $self  ) = shift;
    my( $string ) = shift;
    my( $re );

    $re = $self->_RE();
    return( ( $string =~ m/$re/ ) );

}

# }}} ------------------------------------------------------------

# }}}
# {{{ code: Internal Functions

# {{{  sub: _RE
# {{{   perldoc

=item _RE

Get/set the regular expression that will be used to extract text.
This method is used by Format and FormatMap to generate the required
regex and is essentially used to cache the regex that will be used.

=cut

# }}}

sub _RE {
    my( $self ) = shift;

    $self->{ RE } = shift
      if( scalar( @_ ) );

    return( $self->{ RE } );
}

# }}} ------------------------------------------------------------
# {{{  sub: _convert_format_to_re
# {{{   perldoc

=item _convert_format_to_re

This function parses the format string and generating a regular
expression from it.  It uses %format_map to map symbols in the
format string to regular expressions, by default %format_map
contains the same symbols 

=cut

# }}}

sub _convert_format_to_re {
    my( $format ) = shift;
    my( %format_map ) = @_;
    my( $spec, @formats );

    $spec = join( '', keys( %format_map ) );
    ( @formats ) = ( $format =~ m/%[$spec]/g );
    $format =~
      s{(.*?)%([$spec])}
       { "\Q$1\E" . ( ref( $format_map{ $2 } ) eq "ARRAY"
		      ? join( '|', map( '(' . $_ . ')', @{ $format_map{ $2 } } ) )
		      : "($format_map{ $2 })" ) }ge;

    return( $format );
}

# }}} ------------------------------------------------------------
# {{{  sub: _get_format_order

sub _get_format_order {
    my( $format ) = shift;
    my( %map ) = @_;
    my( @formats ) = keys( %map );

    grep {
	my( $c ) = $_;
	grep( $c eq $_, @formats );
    } ( $format =~ m/%(.)/g );
}

# }}} ------------------------------------------------------------

# }}}

# }}} ------------------------------------------------------------
# {{{ docs: perldocs


=head1 AUTHOR

Mishka Gorodnitzky <misaka@pobox.com>

=head1 COPYRIGHT

This script is licensed under the 'Perl Artistic License'.  See
<http://www.perl.com/pub/language/misc/Artistic.html>.  It comes with
no warranties, expressed or implied.

=cut

# }}}
